/*
 * Support and FAQ: visit <a href="https://www.microchip.com/support/">Microchip Support</a>
 */
#include <asf.h>

//! [buf_length]
#define BUF_LENGTH 20

//! [slave_select_pin]
#define SLAVE_SELECT_PIN CONF_MASTER_SS_PIN


/******************************************************************************
 *  extern/static variables
 *****************************************************************************/

static struct usart_module cdc_uart_module;

//! [buffer]
static uint8_t buffer[BUF_LENGTH] = {
		0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
		0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13
};


//! [dev_inst]
struct spi_module spi_master_instance;

//! [slave_dev_inst]
struct spi_slave_inst slave;


void configure_spi_master(void);


//! [configure_spi]
void configure_spi_master(void)
{
//! [config]
	struct spi_config config_spi_master;

//! [slave_config]
	struct spi_slave_inst_config slave_dev_config;

	/* Configure and initialize software device instance of peripheral slave */
	spi_slave_inst_get_config_defaults(&slave_dev_config);

//! [ss_pin]
	slave_dev_config.ss_pin = SLAVE_SELECT_PIN;

//! [slave_init]
	spi_attach_slave(&slave, &slave_dev_config);

	/* Configure, initialize and enable SERCOM SPI module */
	spi_get_config_defaults(&config_spi_master);

//! [mux_setting]
	config_spi_master.mux_setting = CONF_MASTER_MUX_SETTING;

	config_spi_master.pinmux_pad0 = CONF_MASTER_PINMUX_PAD0;
	config_spi_master.pinmux_pad1 = CONF_MASTER_PINMUX_PAD1;
	config_spi_master.pinmux_pad2 = CONF_MASTER_PINMUX_PAD2;
	config_spi_master.pinmux_pad3 = CONF_MASTER_PINMUX_PAD3;

//! [init]
	spi_init(&spi_master_instance, CONF_MASTER_SPI_MODULE, &config_spi_master);

//! [enable]
	spi_enable(&spi_master_instance);
}



/*********************************************************************
*	Function	: configure_console()
*	Description : This initialize SERCOM USART port for virtual COM-port.
*	Argument	:
*	Return		: None
*********************************************************************/

static void configure_console(void)
{
	struct usart_config usart_conf;

	usart_get_config_defaults(&usart_conf);
	usart_conf.mux_setting = EDBG_CDC_SERCOM_MUX_SETTING;
	usart_conf.pinmux_pad0 = EDBG_CDC_SERCOM_PINMUX_PAD0;
	usart_conf.pinmux_pad1 = EDBG_CDC_SERCOM_PINMUX_PAD1;
	usart_conf.pinmux_pad2 = EDBG_CDC_SERCOM_PINMUX_PAD2;
	usart_conf.pinmux_pad3 = EDBG_CDC_SERCOM_PINMUX_PAD3;
	usart_conf.baudrate    = 115200;

	stdio_serial_init(&cdc_uart_module, EDBG_CDC_MODULE, &usart_conf);
	usart_enable(&cdc_uart_module);
}



int main(void)
{
	system_init();

	configure_console();

	printf("SAMD21 SPI Master Example\r\n");

	configure_spi_master();

	while (true)
	{
		/* Infinite loop */
		if(!port_pin_get_input_level(BUTTON_0_PIN))
		{
			printf("SW0 pressed\r\n");

			//! [select_slave]
			spi_select_slave(&spi_master_instance, &slave, true);

			//! [write]
			spi_write_buffer_wait(&spi_master_instance, buffer, BUF_LENGTH);

			//! [deselect_slave]
			spi_select_slave(&spi_master_instance, &slave, false);

			//! [light_up]
			port_pin_set_output_level(LED_0_PIN, LED0_ACTIVE);
			
			delay_ms(1000);
		}
	}
}
